package com.tuya.iot;

import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;
import java.io.FileInputStream;
import java.net.URL;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

/**
 * 使用自签证书
 * 使用Paho连接Tuya物联网平台
 */
public class ConnectTuyaIotPlatformSelfSignCertDemo {

    public static void main(String[] args) throws Exception {
        // productKey/deviceName/deviceSecret/endpoint replace with your own
        String productKey = "k1ymv1DuLfE";
        String deviceName = "a53b1fa6490e";
        String deviceSecret = "50a0ea208d588e80271125851b4f0d51";
        String endpoint = "si-d6e8d812acb848958054.tuyacloud.com";

        String port = "8883"; //do not change
        String broker = "ssl://" + endpoint+":" + port;
        //计算Mqtt建联参数
        MqttSign sign = new MqttSign();
        sign.calculate(productKey, deviceName, deviceSecret);

        System.out.println("username: " + sign.getUsername());
        System.out.println("password: " + sign.getPassword());
        System.out.println("clientid: " + sign.getClientid());

        MemoryPersistence persistence = new MemoryPersistence();
        try {
            //Paho Mqtt 客户端
            MqttClient sampleClient = new MqttClient(broker, sign.getClientid(), persistence);

            //Paho Mqtt 连接参数
            MqttConnectOptions connOpts = new MqttConnectOptions();
            connOpts.setCleanSession(true);
            connOpts.setKeepAliveInterval(180);
            connOpts.setUserName(sign.getUsername());
            connOpts.setPassword(sign.getPassword().toCharArray());
            connOpts.setSocketFactory(getSSLSocketFactory());

            sampleClient.connect(connOpts);
            System.out.println("broker: " + broker + " Connected");

            //Paho Mqtt 消息订阅
            String topicReply = "/sys/" + productKey + "/" + deviceName + "/thing/event/property/post_reply";
            sampleClient.subscribe(topicReply, new MqttPostPropertyMessageListener());
            System.out.println("subscribe: " + topicReply);

            //Paho Mqtt 消息发布
            String topic = "/sys/" + productKey + "/" + deviceName + "/thing/event/property/post";
            String content = "{\"id\":\"1\",\"version\":\"1.0\",\"params\":{\"LightSwitch\":1}}";
            MqttMessage message = new MqttMessage(content.getBytes());
            message.setQos(0);
            sampleClient.publish(topic, message);
            System.out.println("publish: " + content);

            Thread.sleep(2000);

            //Paho Mqtt 断开连接
            sampleClient.disconnect();
            System.out.println("Disconnected");
            System.exit(0);
        } catch (MqttException e) {
            System.out.println("reason " + e.getReasonCode());
            System.out.println("msg " + e.getMessage());
            System.out.println("loc " + e.getLocalizedMessage());
            System.out.println("cause " + e.getCause());
            System.out.println("excep " + e);
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public static SSLSocketFactory getSSLSocketFactory() throws Exception {
        URL url = ConnectTuyaIotPlatformSelfSignCertDemo.class.getClassLoader().getResource("tuyarootca.crt");
        String fileName = url.getPath();

        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        X509Certificate oCert = (X509Certificate) cf.generateCertificate(new FileInputStream(fileName));

        KeyStore keyStore = KeyStore.getInstance("JKS");
        keyStore.load(null,null);
        keyStore.setCertificateEntry("cam2", oCert);
        TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance("PKIX");
        trustManagerFactory.init(keyStore);

        SSLContext sslContext = SSLContext.getInstance("TLS");
        sslContext.init(null, trustManagerFactory.getTrustManagers(), new SecureRandom());

        return sslContext.getSocketFactory();
    }
}